<?php
/**
 * Listeners
 *
 * @package GamiPress\WooCommerce\Listeners
 * @since 1.0.0
 */

// Exit if accessed directly
if( !defined( 'ABSPATH' ) ) exit;

// Purchase listener
function gamipress_wc_new_purchase( $order_id ) {

    $order = wc_get_order( $order_id );
    $user_id = $order->get_user_id();

    // Bail if not a valid order
    if( ! $order ) return;

    // Trigger new purchase
    do_action( 'gamipress_wc_new_purchase', $order_id, $user_id );

    $items = $order->get_items();

    if ( is_array( $items ) ) {

        // On purchase, trigger events on each product purchased
        foreach ( $items as $item ) {

            $product_id     = 0;
            $variation_id   = 0;
            $quantity       = 1;

            if( class_exists( 'WC_Order_Item' ) && $item instanceof WC_Order_Item ) {

                // WooCommerce >= 3.0.0
                $product_id     = $item->get_product_id();
                $variation_id   = $item->get_variation_id();
                $quantity       = $item->get_quantity();

            } else if( is_array( $item ) && isset( $item['product_id'] ) ) {

                // WooCommerce < 3.0.0
                $product_id     = $item['product_id'];
                $variation_id   = ( isset( $item['variation_id'] ) ? $item['variation_id'] : 0 );
                $quantity       = isset( $item['qty'] ) ? absint( $item['qty'] ) : 1;

            }

            if( $product_id !== 0 ) {

                // Trigger events same times as item quantity
                for ( $i = 0; $i < $quantity; $i++ ) {

                    $vendor_id = absint( get_post_field( 'post_author', $product_id ) );

                    if( $vendor_id !== 0 ) {
                        // Trigger new product sale to award the vendor
                        do_action( 'gamipress_wc_new_sale', $product_id, $vendor_id, $order_id, $quantity );
                    }

                    // Trigger new product purchase
                    do_action( 'gamipress_wc_new_product_purchase', $product_id, $user_id, $order_id, $quantity );

                    // Trigger specific product purchase
                    do_action( 'gamipress_wc_specific_product_purchase', $product_id, $user_id, $order_id, $quantity );

                    if( $variation_id !== 0 ) {
                        // Trigger specific product variation purchase
                        do_action( 'gamipress_wc_product_variation_purchase', $product_id, $user_id, $variation_id, $order_id, $quantity );
                    }

                    // Get an array of categories IDs attached to the product
                    $categories = wc_get_product_term_ids( $product_id, 'product_cat' );

                    if( ! empty( $categories ) ) {

                        foreach( $categories as $category_id ) {
                            // Trigger specific product category purchase (trigger 1 event per category)
                            do_action( 'gamipress_wc_product_category_purchase', $product_id, $user_id, $category_id, $order_id, $quantity );
                        }

                    }

                    // Get an array of tags IDs attached to the product
                    $tags = wc_get_product_term_ids( $product_id, 'product_tag' );

                    if( ! empty( $tags ) ) {

                        foreach( $tags as $tag_id ) {
                            // Trigger specific product tag purchase (trigger 1 event per tag)
                            do_action( 'gamipress_wc_product_tag_purchase', $product_id, $user_id, $tag_id, $order_id, $quantity );
                        }

                    }

                }

            }

        } // end foreach

    } // end if $items is an array

}
add_action( 'woocommerce_payment_complete', 'gamipress_wc_new_purchase' );

// Refund purchase listener
function gamipress_wc_purchase_refund( $order_id ) {

    $order = wc_get_order( $order_id );
    $user_id = $order->get_user_id();

    if( $order ) {
        // Trigger purchase refund
        do_action( 'gamipress_wc_purchase_refund', $order_id, $user_id );

        $items = $order->get_items();

        if ( is_array( $items ) ) {

            // On purchase, trigger events on each product purchased
            foreach ( $items as $item ) {

                $product_id     = 0;
                $variation_id   = 0;
                $quantity       = 1;

                if( class_exists( 'WC_Order_Item' ) && $item instanceof WC_Order_Item ) {

                    // WooCommerce >= 3.0.0
                    $product_id     = $item->get_product_id();
                    $variation_id   = $item->get_variation_id();
                    $quantity       = $item->get_quantity();

                } else if( is_array( $item ) && isset( $item['product_id'] ) ) {

                    // WooCommerce < 3.0.0
                    $product_id     = $item['product_id'];
                    $variation_id   = ( isset( $item['variation_id'] ) ? $item['variation_id'] : 0 );
                    $quantity       = isset( $item['qty'] ) ? absint( $item['qty'] ) : 1;

                }

                if( $product_id !== 0 ) {

                    // Trigger events same times as item quantity
                    for ( $i = 0; $i < $quantity; $i++ ) {

                        $vendor_id = absint( get_post_field( 'post_author', $product_id ) );

                        if( $vendor_id !== 0 ) {
                            // Trigger product refunded to award the vendor
                            do_action( 'gamipress_wc_user_product_refund', $product_id, $vendor_id, $order_id, $quantity );
                        }

                        // Trigger product refund
                        do_action( 'gamipress_wc_product_refund', $product_id, $user_id, $order_id, $quantity );

                        // Trigger specific product refund
                        do_action( 'gamipress_wc_specific_product_refund', $product_id, $user_id, $order_id, $quantity );

                        if( $variation_id !== 0 ) {

                            // Trigger specific product variation refund
                            do_action( 'gamipress_wc_product_variation_refund', $product_id, $user_id, $variation_id, $order_id, $quantity );

                        }

                        // Get an array of categories IDs attached to the product
                        $categories = wc_get_product_term_ids( $product_id, 'product_cat' );

                        if( ! empty( $categories ) ) {

                            foreach( $categories as $category_id ) {
                                // Trigger specific product category refund (trigger 1 event per category)
                                do_action( 'gamipress_wc_product_category_refund', $product_id, $user_id, $category_id, $order_id, $quantity );
                            }

                        }

                        // Get an array of tags IDs attached to the product
                        $tags = wc_get_product_term_ids( $product_id, 'product_tag' );

                        if( ! empty( $tags ) ) {

                            foreach( $tags as $tag_id ) {
                                // Trigger specific product tag refund (trigger 1 event per tag)
                                do_action( 'gamipress_wc_product_tag_refund', $product_id, $user_id, $tag_id, $order_id, $quantity );
                            }

                        }

                    }

                }

            } // end foreach

        } // end if $items is an array

    } // end if $order

}

// Check order status changes to meet if should award to the user
function gamipress_wc_check_order_status_change( $order_id, $from, $to, $order ) {

    if( $from !== 'completed' && $to === 'completed' )
        gamipress_wc_new_purchase( $order_id );

    if( $from !== 'refunded' && $to === 'refunded' )
        gamipress_wc_purchase_refund( $order_id );

}
add_action( 'woocommerce_order_status_changed', 'gamipress_wc_check_order_status_change', 10, 4 );

// Review listener
function gamipress_wc_approved_review_listener( $comment_ID, $comment ) {

    // Enforce array for both hooks (wp_insert_comment uses object, comment_{status}_comment uses array)
    if ( is_object( $comment ) ) {
        $comment = get_object_vars( $comment );
    }

    $comment_ID = absint( $comment_ID );
    $user_id = absint( $comment[ 'user_id' ] );
    $post_id = absint( $comment[ 'comment_post_ID' ] );
    $post = get_post( $post_id );

    // Check if comment is a review
    // In some release, WooCommerce stop to set the comment_type as review and now reviews are based on assigned post ID post_type
    //if ( $comment[ 'comment_type' ] !== 'review' ) {
    if ( $post->post_type !== 'product' ) {
        return;
    }

    // Check if review is approved
    if ( 1 !== (int) $comment[ 'comment_approved' ] ) {
        return;
    }

    // Trigger review actions
    do_action( 'gamipress_wc_new_review', $comment_ID, $user_id, $post_id, $comment );
    do_action( 'gamipress_wc_specific_new_review', $comment_ID, $user_id, $post_id, $comment );

    if ( absint( $post->post_author ) !== 0 ) {
        // Trigger get review actions to product author
        do_action( 'gamipress_wc_get_review', $comment_ID, absint( $post->post_author ), $post_id, $user_id, $comment );
        do_action( 'gamipress_wc_get_specific_review', $comment_ID, absint( $post->post_author ), $post_id, $user_id, $comment );
    }

}
add_action( 'comment_approved_', 'gamipress_wc_approved_review_listener', 10, 2 );
add_action( 'comment_approved_review', 'gamipress_wc_approved_review_listener', 10, 2 );
add_action( 'wp_insert_comment', 'gamipress_wc_approved_review_listener', 10, 2 );

/**
 * Purchase subscription listener
 *
 * @since 1.0.0
 *
 * @param WC_Subscription $subscription
 */
function gamipress_wc_subscription_purchase_listener( $subscription ) {

    $items = $subscription->get_items();

    // Bail if no items purchased
    if ( ! is_array( $items ) ) {
        return;
    }

    $user_id = $subscription->get_user_id();

    // Loop all items to trigger events on each one purchased
    foreach ( $items as $item ) {

        $product_id     = $item->get_product_id();
        $quantity       = $item->get_quantity();

        // Skip items not assigned to a product
        if( $product_id === 0 ) {
            continue;
        }

        // Trigger events same times as item quantity
        for ( $i = 0; $i < $quantity; $i++ ) {

            // Any product subscription purchase
            do_action( 'gamipress_wc_subscription_purchase', $product_id, $user_id, $subscription->get_id() );

            // Specific product subscription purchase
            do_action( 'gamipress_wc_specific_subscription_purchase', $product_id, $user_id, $subscription->get_id() );


        } // End for of quantities

    } // End foreach of items


}
add_action( 'woocommerce_subscription_payment_complete', 'gamipress_wc_subscription_purchase_listener' );

/**
 * Subscription renewal listener
 *
 * @since 1.0.0
 *
 * @param WC_Subscription $subscription
 */
function gamipress_wc_subscription_renewal_listener( $subscription ) {

    $items = $subscription->get_items();

    // Bail if no items purchased
    if ( ! is_array( $items ) ) {
        return;
    }

    $user_id = $subscription->get_user_id();

    // Loop all items to trigger events on each one purchased
    foreach ( $items as $item ) {

        $product_id     = $item->get_product_id();
        $quantity       = $item->get_quantity();

        // Skip items not assigned to a product
        if( $product_id === 0 ) {
            continue;
        }

        // Trigger events same times as item quantity
        for ( $i = 0; $i < $quantity; $i++ ) {

            // Any product subscription renewal
            do_action( 'gamipress_wc_subscription_renewal', $product_id, $user_id, $subscription->get_id() );

            // Specific product subscription renewal
            do_action( 'gamipress_wc_specific_subscription_renewal', $product_id, $user_id, $subscription->get_id() );


        } // End for of quantities

    } // End foreach of items

}
add_action( 'woocommerce_subscription_renewal_payment_complete', 'gamipress_wc_subscription_renewal_listener' );

/**
 * Subscription status updated listener
 *
 * @since 1.0.0
 *
 * @param WC_Subscription $subscription
 */
function gamipress_wc_subscription_status_updated_listener( $subscription ) {

    $status = false;

    if( $subscription->has_status( array( 'pending-cancel', 'cancelled' ) ) ) {
        $status = 'cancelled';
    } else if( $subscription->has_status( array( 'expired' ) ) ) {
        $status = 'expired';
    }

    if( $status === false ) {
        return;
    }

    $items = $subscription->get_items();

    // Bail if no items purchased
    if ( ! is_array( $items ) ) {
        return;
    }

    $user_id = $subscription->get_user_id();

    // Loop all items to trigger events on each one purchased
    foreach ( $items as $item ) {

        $product_id     = $item->get_product_id();
        $quantity       = $item->get_quantity();

        // Skip items not assigned to a product
        if( $product_id === 0 ) {
            continue;
        }

        // Trigger events same times as item quantity
        for ( $i = 0; $i < $quantity; $i++ ) {

            // Any product subscription cancelled or expired
            do_action( "gamipress_wc_subscription_{$status}", $product_id, $user_id, $subscription->get_id() );

            // Specific product subscription cancelled or expired
            do_action( "gamipress_wc_specific_subscription_{$status}", $product_id, $user_id, $subscription->get_id() );


        } // End for of quantities

    } // End foreach of items

}
add_action( 'woocommerce_subscription_status_updated', 'gamipress_wc_subscription_status_updated_listener' );